/* 
 * mscaleAR.c --- written by Yasuko Matsubara 2012.6.7
 */


#include        <stdio.h>
#include        <string.h>
#include        <stdlib.h>
#include        <math.h>
#define   BUFSIZE            256
#define MAXORDERS   100
#define MAXDIM      100
#include	"nrutil.h"
#include "tool.h"

#define NORMALIZE 0
#define LOGARITHM 0 //1

void _load_orders();
void _malloc_read();
void _smoothingMatrix();
void _makeMatrix_cross();
void _makeMatrix();
void _callRLS();
void _generate_cross();
void _writePsy();

typedef struct {

  int K ; //#latent topics
  int w1, w2, w3, wc;

  int order;
  int order_w1;
  int order_w2;
  int order_w3;
  int order_wc;

  int duration;     //length of sequences
  int gen_length;   //length of "gen"-sequence
  double **psy1;    //[K][duration]
  double **psy2;    //[K][duration]
  double **psy3;    //[K][duration]
  double **M;       //[duration][order+1]
  double **coeff;   //[K][order]
  double **predict; //[K][duration+genlen]

} ARModel;



int main(int argc, char* argv[]){


  if(argc != 7) 
	error("Usage: ./cmd $dat $dim $dur $genlen $orderfn $outdir", argv[0]);


  char* psyfn = argv[1];
  int dim = atoi(argv[2]);
  int duration = atoi(argv[3]);
  int genlen = atoi(argv[4]);
  char *orderfn = argv[5];
  char *outfn = argv[6];

  ARModel model;
  model.K = dim;
  model.duration = duration;
  model.gen_length = genlen;

  /// loading
  _load_orders(orderfn, &model);
  _malloc_read(psyfn, &model);
  /// setting
  _smoothingMatrix(&model);

  int k;

  /// learn models
  for(k=0;k<model.K;k++){
      _makeMatrix(&model, k);
      rls(model.M, model.order, model.duration, model.coeff[k]);
  }//k

  /// predict sequences
  _generate_cross(&model);
  /// output data
  _writePsy(outfn, &model);

  //return 0;
}

void _smoothPsy(double *org, int n, double *smooth, int wdsize){
  int t,w;
  int n_s= (int)(n/wdsize);
  for(t=0;t<n_s;t++){
    smooth[t]=0;
    for(w=0;w<wdsize;w++)
      smooth[t] += org[t*wdsize+w];
    smooth[t] /= wdsize;
  }//t


}
void _smoothingMatrix(ARModel *model){
  int k,t,w;
  for(k=0;k<model->K;k++){
    _smoothPsy(model->psy1[k], model->duration, model->psy2[k], model->w2);
    _smoothPsy(model->psy1[k], model->duration, model->psy3[k], model->w3);
  }//k

}

void _makeMatrix(ARModel *model, int dimID){

  int i,j,t,t1,t2,t3,c,k;

  int w1 = model->w1;
  int w2 = model->w2;
  int w3 = model->w3;
  int wc = model->wc;
 
  int C1 = model->order_w1;
  int C2 = model->order_w2;
  int C3 = model->order_w3;
  int Cc = model->order_wc;
  double **psy1 = model->psy1;
  double **psy2 = model->psy2;
  double **psy3 = model->psy3;
  double    **M = model->M;

  for(t=0; t<model->duration;t++){

    if( (t-C1*w1) < 0 ) continue;
    if( (t-C2*w2) < 0 ) continue;  
    if( (t-C3*w3) < 0 ) continue;
    if( (t-Cc*wc) < 0 ) continue;

    int loc = 0;
    //WC
    for(c=0; c<Cc;c++){
      t1 = (int) (t/w1);
      for(k=0;k<model->K;k++){
        M[t][loc]=psy1[k][t1-Cc+c];
        loc++;
      }//k
    }//c
    //W1
    for(c=0; c<C1; c++){
      t1= (int) (t/w1);
      M[t][loc]=psy1[dimID][t1-C1+c];
      loc++;
    }//c
    //W2
    for(c=0; c<C2; c++){
      t2 = (int) (t/w2);
      M[t][loc]=psy2[dimID][t2-C2+c];
      loc++;
    }//c
    //W3
    for(c=0; c<C3; c++){
      t3 = (int) (t/w3);
      M[t][loc]=psy3[dimID][t3-C3+c];
      loc++;
    }//c
    //latest
    M[t][loc]=psy1[dimID][t];
  }//t

}



void _generate_cross(ARModel *model){

  int i,j,t,t1,t2,t3,dim;
  double sum;
  int c,k,w;

  int w1 = model->w1;
  int w2 = model->w2;
  int w3 = model->w3;
  int wc = model->wc;

  int C1 = model->order_w1;
  int C2 = model->order_w2;
  int C3 = model->order_w3;
  int Cc = model->order_wc;

  double *coeff;
  double **predict;
  predict = model->predict;

  /// padding current sequences
  for(t=0;t<model->duration;t++)
    for(dim=0;dim<model->K;dim++)
      predict[dim][t] = model->psy1[dim][t];

  /// predict each dims at time t
  int ed = model->duration+ model->gen_length;
  for(t=model->duration;t<ed;t++){
    for(dim=0;dim<model->K;dim++){

      ///setting
      predict[dim][t] = 0;
      coeff = model->coeff[dim];

      int loc=0;

      //WC
      for(c=0;c<Cc;c++)
        for(k=0;k<model->K;k++){
          predict[dim][t] += coeff[loc] * predict[k][t-Cc+c];
          loc++;
        }//k
      //W1
      for(c=0; c<C1; c++){
        predict[dim][t] += coeff[loc] * predict[dim][t-C1+c];
        loc++;
      }//c

      //W2
      for(c=0; c<C2; c++){
        t2 = (int) (t/w2);
        double w2val = 0.0;
        for(w=0;w<w2;w++)
          w2val += predict[dim][w2*(t2-C2+c)+w];
        w2val /= w2;
        predict[dim][t] += coeff[loc] * w2val;
        loc++;
      }//c

      //W3
      for(c=0; c<C3; c++){
        t3 = (int) (t/w3);
        double w3val = 0.0;
        for(w=0;w<w3;w++)
          w3val += predict[dim][w3*(t3-C3+c)+w];
        w3val /= w3;
        predict[dim][t] += coeff[loc] * w3val;
        loc++;
      }//c

    if(predict[dim][t]<0) predict[dim][t] = 0;
#if(NORMALIZE)
    if(predict[dim][t]>1) predict[dim][t] = 1;
#endif
  }//dim


#if(NORMALIZE)
  ///normalize
  sum = 0.0;
  for(dim=0;dim<model->K;dim++)
    sum += predict[url][dim][t];
  for(dim=0;dim<model->K;dim++)
    predict[url][dim][t]/=sum;

  if(sum==0)
    for(dim=0;dim<model->K;dim++)
      predict[url][dim][t] = 0;
#endif


  }//t

}


void _load_orders(char *fn, ARModel *model){
  int i,j;
  char cbuf[BUFSIZE];
  int ibuf;
  FILE *fp;
  if((fp = fopen(fn,"r")) == NULL)
    error("cannot open:", fn);
  fscanf(fp, "%s %d %d", cbuf, &model->w1, &model->order_w1);
  fscanf(fp, "%s %d %d", cbuf, &model->w2, &model->order_w2);
  fscanf(fp, "%s %d %d", cbuf, &model->w3, &model->order_w3);
  fscanf(fp, "%s %d %d", cbuf, &model->wc, &model->order_wc);
  fclose(fp);
  model->order = 	    model->order_w1 
	+ model->order_w2 + model->order_w3 
	+ model->order_wc * model->K;

  fprintf(stderr, "-----orders----\n");
  fprintf(stderr, "W1: %d, %d\n", model->w1, model->order_w1);
  fprintf(stderr, "W2: %d, %d\n", model->w2, model->order_w2);
  fprintf(stderr, "W3: %d, %d\n", model->w3, model->order_w3);
  fprintf(stderr, "WC: %d, %d\n", model->wc, model->order_wc);
}

void _readpsy(char *fn, ARModel *model){
  int i,t;
  FILE *fp;
  if((fp = fopen(fn,"r")) == NULL)
    error("cannot open :", fn);

  float fbuf;
  for(t=0;t<model->duration;t++)
    for( i=0;i<model->K;i++){
      fscanf(fp, "%f", &fbuf);
      model->psy1[i][t] = fbuf;
      if(model->psy1[i][t] < 0) 
        model->psy1[i][t] = 0;
      #if(LOGARITHM)
      model->psy1[i][t]=log(model->psy1[i][t]+1);
      #endif
    }//i
  fclose(fp);
}
void _malloc_read(char *psyfn, ARModel *model){
  model->coeff = (double**) dmatrix(0,model->K, 0, model->order);
  model->psy1  = (double**) dmatrix(0,model->K, 0, model->duration);
  model->psy2  = (double**) dmatrix(0,model->K, 0, model->duration);
  model->psy3  = (double**) dmatrix(0,model->K, 0, model->duration);
  model->M     = (double**) dmatrix(0,model->duration, 0, model->order+1);
  model->predict=(double**) dmatrix(0,model->K, 0, model->duration+model->gen_length);
  _readpsy(psyfn, model);
}

void _writePsy(char *fn, ARModel *model){
    int i,t;
    FILE *fp;
    float fbuf;
    if((fp = fopen(fn,"w")) == NULL)
      error("cannot open:", fn);
    for(t=0;t<model->duration+model->gen_length;t++){
      for( i=0;i<model->K;i++)
        #if(LOGARITHM)
        fprintf(fp, "%f ", exp(model->predict[i][t])-1);
        #endif
        #if(!LOGARITHM)
        fprintf(fp, "%f ", model->predict[i][t]);
        #endif
      fprintf(fp, "\n");
    }//t
    fclose(fp);
}


