/* 
 * tool.c --- written by yasuko matsubara 2012.6.7
 */

#include <stdio.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <sys/time.h>
#include <sys/resource.h>
#include "trimine.h"
#include "nrutil.h"

void readMatrix();
void readfMatrix();
void readiMatrix();
void readfVector();
void readiVector();

void writeMatrix();
void writefMatrix();
void writeiMatrix();
void write_f_twd();
void write_f_xijt();
void malloc_read();
int count_lines();
double getrusage_sec();
void notfin();
void error();

void readMatrix(char* fn, double **M, int x, int y){
// M[x][y]
//
// M[1...x]
//  [.    ]
//  [y....]

  int i,j;
  FILE *fp;
  float fbuf;
  if((fp=fopen(fn,"r"))==NULL)
    error("cannot open",fn);
  for(j=0;j<y;j++){
    for(i=0;i<x;i++){
      fscanf(fp, "%f", &fbuf);
      M[i][j]=(double)fbuf;
    }//i
  }//j
  fclose(fp);

}
void readfMatrix(char* fn, float **M, int x, int y){
// M[x][y]
//
// M[1...x]
//  [.    ]
//  [y....]

  int i,j;
  FILE *fp;
  float fbuf;
  if((fp=fopen(fn,"r"))==NULL)
    error("cannot open",fn);
  for(j=0;j<y;j++){
    for(i=0;i<x;i++){
      fscanf(fp, "%f", &fbuf);
      M[i][j]=(float)fbuf;
    }//i
  }//j
  fclose(fp);

}
void readiVector(char* fn, int *V, int n){
// int V[n]
  int i;
  FILE *fp;
  float fbuf;
  if((fp=fopen(fn,"r"))==NULL)
    error("cannot open",fn);
  for(i=0;i<n;i++){
    fscanf(fp, "%f", &fbuf);
    V[i]=(int)fbuf;
  }//i
  fclose(fp);
}
void readfVector(char* fn, float *V, int n){
// float V[n]
  int i;
  FILE *fp;
  float fbuf;
  if((fp=fopen(fn,"r"))==NULL)
    error("cannot open",fn);
  for(i=0;i<n;i++){
    fscanf(fp, "%f", &fbuf);
    V[i]=fbuf;
  }//i
  fclose(fp);
}
void readiMatrix(char* fn, int **M, int x, int y){
// M[x][y]
//
// M[1...x]
//  [.    ]
//  [y....]

  int i,j;
  FILE *fp;
  float fbuf;
  if((fp=fopen(fn,"r"))==NULL)
    error("cannot open",fn);
  for(j=0;j<y;j++){
    for(i=0;i<x;i++){
      fscanf(fp, "%f", &fbuf);
      M[i][j]=(int)fbuf;
    }//i
  }//j
  fclose(fp);

}
void writefMatrix(char* fn, float **M, int x, int y){
// M[x][y]
//
// M[1...x]
//  [.    ]
//  [y....]

  int i,j;
  FILE *fp;

  if((fp=fopen(fn,"w"))==NULL)
    error("cannot open",fn);
  for(j=0;j<y;j++){
    for(i=0;i<x;i++)
      fprintf(fp, "%f ", M[i][j]);
    fprintf(fp, "\n");
  }//j
  fclose(fp);

}

void writeMatrix(char* fn, double **M, int x, int y){
// M[x][y]
//
// M[1...x]
//  [.    ]
//  [y....]

  int i,j;
  FILE *fp;

  if((fp=fopen(fn,"w"))==NULL)
    error("cannot open",fn);
  for(j=0;j<y;j++){
    for(i=0;i<x;i++)
      fprintf(fp, "%f ", M[i][j]);
    fprintf(fp, "\n");
  }//j
  fclose(fp);

}
void writeiMatrix(char* fn, int **M, int x, int y){
// M[x][y]
//
// M[1...x]
//  [.    ]
//  [y....]

  int i,j;
  FILE *fp;

  if((fp=fopen(fn,"w"))==NULL)
    error("cannot open",fn);
  for(j=0;j<y;j++){
    for(i=0;i<x;i++)
      fprintf(fp, "%d ", M[i][j]);
    fprintf(fp, "\n");
  }//j
  fclose(fp);

}
void write_f_xijt(char* outdir, float ***x_ijt, int u, int v, int n){
   int i;
   char fn[BUFSIZE];
   // x_ijt[u][v][n]
   // write x_ijt
   for(i=0;i<u;i++){
     sprintf(fn, "%sgen.url%d", outdir, i+1); 
     writefMatrix(fn, x_ijt[i], v, n);
   }//i
}


void write_f_twd(char* outdir, float ***M, int u, int v, int n){
// M[u][v][n]
//
// dat.t1, dat.t2, ..., dat.tn
//
// u v 
// 3 1:1 2:5 6:9 1 1
// 2 4:9 6:1 1 1
// 5 3:2 5:1 ....
//

  int i,j,t;
  FILE *fp;
  char fn[BUFSIZE];
  for(t=0;t<n;t++){
    sprintf(fn, "%sdat.t%d", outdir, t+1);
    if((fp=fopen(fn,"w"))==NULL)
      error("cannot open",fn);
    fprintf(fp, "%d\t%d\t%d\n", u, v, 0);
    for(i=0;i<u;i++){
      int cnt = 0;
      for(j=0;j<v;j++)
        if(M[i][j][t]>0)
          cnt++;
      fprintf(fp, "%d ", cnt);
      for(j=0;j<v;j++)
        if(M[i][j][t]>0)
          fprintf(fp, "%d:%d ", j+1, (int) (M[i][j][t]+0.5));
      fprintf(fp, "1 1 \n");
    }//j
    fclose(fp);
  }//t
}

void _read_slice(char* fn, DATA *data, int u, int v){

  FILE *fp;
  int i,j, buf;
  if((fp = fopen(fn,"r")) == NULL)
    error("cannot open",fn);
  data->Mx = (int*) ivector(0, u);
  data->x = (int**)malloc(sizeof(int*)*u);
  data->ind = (int**)malloc(sizeof(int*)*u);
  if (data->x == NULL || data->ind == NULL)
    error("Allocation error","read_slice()");

  fscanf(fp, "%d %d %d", &buf, &buf, &buf);
  for(j=0; j<u; j++){
    fscanf(fp, "%d", &data->Mx[j]);
    data->ind[j] = (int*) ivector(0, data->Mx[j]);
    data->x[j]   = (int*) ivector(0, data->Mx[j]);
    for(i=0; i<data->Mx[j]; i++){
      fscanf(fp, "%d:%d", &data->ind[j][i], &data->x[j][i]);
      data->ind[j][i]--;
    }//i
    fscanf(fp, "%d %d", &buf, &buf);
  }//j
  fclose(fp);

  data->NM = 0;
  for(i=0;i<u;i++)
    for(j=0;j<data->Mx[i];j++)
      data->NM += data->x[i][j] ;
}

void malloc_read(char* fn, TRIMINE *model){
  int i, total=0;
  FILE *fp;
  char buf[BUFSIZE];
  if((fp = fopen(fn,"r")) == NULL)
    error("cannot open",fn);

  model->tensor=(DATA*)malloc(sizeof(DATA)*model->n);
  for(i=0;i<model->n;i++){
    fscanf(fp, "%s", buf);
    _read_slice(buf, &model->tensor[i], model->u, model->v);
    total+=model->tensor[i].NM;
  }//i
  model->NM=total;
  fprintf(stderr, "NM (total # of events) = %d\n", total);
}



///------------------------------------------------///
///
/// others
///
///------------------------------------------------///



//---------------------------------//
//count_lines
//---------------------------------//
int count_lines(filename)
char *filename;
{
  int j;
  char  read_buf[BUFSIZE];
  FILE  *fp;

  if ( ( fp = fopen ( filename , "r" ) ) == NULL ){
    fprintf(stderr, "  %s can not open\n" , filename);
    exit(1);
  }
  j = 0;
  while (fgets(read_buf,BUFSIZE,fp) != NULL ){
    j++;
  }
  fclose(fp);
  return j;
}

//---------------------------------//
// getrusage_sec()
//---------------------------------//
double getrusage_sec(){
    struct rusage t;
    struct timeval tv;
    getrusage(RUSAGE_SELF, &t);
    tv = t.ru_utime;
    return tv.tv_sec + (double)tv.tv_usec*1e-6;
}


void error(char *msg, char *name){
        fprintf(stderr, "error: %s [%s] \n", msg, name);
	exit(1);
}

